@@ -33,24 +33,20 @@
 #include <string.h>
 #include <gconf/gconf-client.h>
 
-#include <libhal-gmanager.h>
+#include <hal-manager.h>
 
 #include "gpm-tray-icon.h"
 #include "gpm-common.h"
 #include "gpm-prefs.h"
-#include "gpm-conf.h"
 #include "gpm-prefs-core.h"
 #include "egg-debug.h"
 #include "gpm-stock-icons.h"
-#include "gpm-screensaver.h"
 #include "gpm-prefs-server.h"
 
 #ifdef HAVE_GCONF_DEFAULTS
 #include <polkit-gnome/polkit-gnome.h>
 #endif
 
-static void     gpm_prefs_class_init (GpmPrefsClass *klass);
-static void     gpm_prefs_init       (GpmPrefs      *prefs);
 static void     gpm_prefs_finalize   (GObject	    *object);
 
 #define GPM_PREFS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_PREFS, GpmPrefsPrivate))
@@ -67,8 +63,8 @@ struct GpmPrefsPrivate
 	gboolean		 can_shutdown;
 	gboolean		 can_suspend;
 	gboolean		 can_hibernate;
-	GpmConf			*conf;
-	GpmScreensaver		*screensaver;
+	guint			 idle_delay;
+	GConfClient		*conf;
 #ifdef HAVE_GCONF_DEFAULTS
 	PolKitGnomeAction	*default_action;
 #endif
@@ -92,8 +88,11 @@ G_DEFINE_TYPE (GpmPrefs, gpm_prefs, G_TYPE_OBJECT)
 #define ACTION_BLANK_TEXT		_("Blank screen")
 #define ACTION_NOTHING_TEXT		_("Do nothing")
 
-/* If sleep time in a slider is set to 61 it is considered as never */
-const int NEVER_TIME_ON_SLIDER = 61;
+/* If sleep time in a slider is set to 122 it is considered as never.
+ * We use 122 because gnome-screensaver's idle delay can be pushed to 120.
+ * We leave ourselves an extra minute past that (because our slider is always
+ * at least idle_delay+1).  Then 122 is the special value 'never'. */
+const int NEVER_TIME_ON_SLIDER = 122;
 
 /**
  * gpm_prefs_class_init:
@@ -136,7 +135,7 @@ void
 gpm_prefs_activate_window (GpmPrefs *prefs)
 {
 	GtkWidget *widget;
-	widget = glade_xml_get_widget (prefs->priv->glade_xml, "window_preferences");
+	widget = glade_xml_get_widget (prefs->priv->glade_xml, "dialog_preferences");
 	gtk_window_present (GTK_WINDOW (widget));
 }
 
@@ -255,7 +254,7 @@ gpm_prefs_icon_radio_cb (GtkWidget *widget,
 	policy = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "policy"));
 	str = gpm_tray_icon_mode_to_string (policy);
 	egg_debug ("Changing %s to %s", GPM_CONF_UI_ICON_POLICY, str);
-	gpm_conf_set_string (prefs->priv->conf, GPM_CONF_UI_ICON_POLICY, str);
+	gconf_client_set_string (prefs->priv->conf, GPM_CONF_UI_ICON_POLICY, str, NULL);
 }
 
 /**
@@ -300,7 +299,6 @@ gpm_prefs_sleep_slider_changed_cb (GtkRange *range,
 				   GpmPrefs *prefs)
 {
 	int value;
-	int gs_idle_time;
 	char *gpm_pref_key;
 
 	value = (int) gtk_range_get_value (range);
@@ -310,10 +308,9 @@ gpm_prefs_sleep_slider_changed_cb (GtkRange *range,
 		value = 0;
 	} else {
 		/* We take away the g-s idle time as the slider represents
-		 * global time but we only do our timeout from when g-s
+		 * global time but we only do our timeout from when gnome-session
 		 * declares the session idle */
-		gs_idle_time = gpm_screensaver_get_delay (prefs->priv->screensaver);
-		value -= gs_idle_time;
+		value -= prefs->priv->idle_delay;
 
 		/* policy is in seconds, slider is in minutes */
 		value *= 60;
@@ -321,7 +318,7 @@ gpm_prefs_sleep_slider_changed_cb (GtkRange *range,
 
 	gpm_pref_key = (char *) g_object_get_data (G_OBJECT (range), "conf_key");
 	egg_debug ("Changing %s to %i", gpm_pref_key, value);
-	gpm_conf_set_int (prefs->priv->conf, gpm_pref_key, value);
+	gconf_client_set_int (prefs->priv->conf, gpm_pref_key, value, NULL);
 }
 
 /**
@@ -331,21 +328,19 @@ gpm_prefs_sleep_slider_changed_cb (GtkRange *range,
  * @gpm_pref_key: The GConf key for this preference setting.
  **/
 static GtkWidget *
-gpm_prefs_setup_sleep_slider (GpmPrefs    *prefs,
-			      const gchar *widget_name,
-			      const gchar *gpm_pref_key)
+gpm_prefs_setup_sleep_slider (GpmPrefs *prefs, const gchar *widget_name, const gchar *gpm_pref_key)
 {
 	GtkWidget *widget;
 	gint value;
 	gboolean is_writable;
-	guint gs_idle_time;
+	GtkAdjustment *adjustment;
 
 	widget = glade_xml_get_widget (prefs->priv->glade_xml, widget_name);
 	g_signal_connect (G_OBJECT (widget), "format-value",
 			  G_CALLBACK (gpm_prefs_format_time_cb), prefs);
 
-	gpm_conf_get_int (prefs->priv->conf, gpm_pref_key, &value);
-	gpm_conf_is_writable (prefs->priv->conf, gpm_pref_key, &is_writable);
+	value = gconf_client_get_int (prefs->priv->conf, gpm_pref_key, NULL);
+	is_writable = gconf_client_key_is_writable (prefs->priv->conf, gpm_pref_key, NULL);
 
 	gtk_widget_set_sensitive (widget, is_writable);
 
@@ -354,10 +349,13 @@ gpm_prefs_setup_sleep_slider (GpmPrefs    *prefs,
 	} else {
 		/* policy is in seconds, slider is in minutes */
 		value /= 60;
-		gs_idle_time = gpm_screensaver_get_delay (prefs->priv->screensaver);
-		value += gs_idle_time;
+		value += prefs->priv->idle_delay;
 	}
 
+	/* set upper */
+	adjustment = gtk_range_get_adjustment (GTK_RANGE (widget));
+	gtk_adjustment_set_upper (adjustment, NEVER_TIME_ON_SLIDER);
+
 	gtk_range_set_value (GTK_RANGE (widget), value);
 
 	g_object_set_data (G_OBJECT (widget), "conf_key", (gpointer) gpm_pref_key);
@@ -386,7 +384,7 @@ gpm_prefs_brightness_slider_changed_cb (GtkRange *range,
 
 	g_object_set_data (G_OBJECT (range), "conf_key", (gpointer) gpm_pref_key);
 	egg_debug ("Changing %s to %i", gpm_pref_key, (int) value);
-	gpm_conf_set_int (prefs->priv->conf, gpm_pref_key, (gint) value);
+	gconf_client_set_int (prefs->priv->conf, gpm_pref_key, (gint) value, NULL);
 }
 
 /**
@@ -410,8 +408,8 @@ gpm_prefs_setup_brightness_slider (GpmPrefs    *prefs,
 	g_signal_connect (G_OBJECT (widget), "format-value",
 			  G_CALLBACK (gpm_prefs_format_percentage_cb), NULL);
 
-	gpm_conf_get_int (prefs->priv->conf, gpm_pref_key, &value);
-	gpm_conf_is_writable (prefs->priv->conf, gpm_pref_key, &is_writable);
+	value = gconf_client_get_int (prefs->priv->conf, gpm_pref_key, NULL);
+	is_writable = gconf_client_key_is_writable (prefs->priv->conf, gpm_pref_key, NULL);
 
 	gtk_widget_set_sensitive (widget, is_writable);
 
@@ -459,7 +457,7 @@ gpm_prefs_action_combo_changed_cb (GtkWidget *widget,
 	g_free (value);
 	gpm_pref_key = (char *) g_object_get_data (G_OBJECT (widget), "conf_key");
 	egg_debug ("Changing %s to %s", gpm_pref_key, action);
-	gpm_conf_set_string (prefs->priv->conf, gpm_pref_key, action);
+	gconf_client_set_string (prefs->priv->conf, gpm_pref_key, action, NULL);
 }
 
 /**
@@ -484,8 +482,8 @@ gpm_prefs_setup_action_combo (GpmPrefs     *prefs,
 
 	widget = glade_xml_get_widget (xml, widget_name);
 
-	gpm_conf_get_string (prefs->priv->conf, gpm_pref_key, &value);
-	gpm_conf_is_writable (prefs->priv->conf, gpm_pref_key, &is_writable);
+	value = gconf_client_get_string (prefs->priv->conf, gpm_pref_key, NULL);
+	is_writable = gconf_client_key_is_writable (prefs->priv->conf, gpm_pref_key, NULL);
 
 	gtk_widget_set_sensitive (widget, is_writable);
 
@@ -559,7 +557,7 @@ gpm_prefs_checkbox_lock_cb (GtkWidget *widget,
 	widget_name = gtk_widget_get_name (widget);
 	gpm_pref_key = (char *) g_object_get_data (G_OBJECT (widget), "conf_key");
 	egg_debug ("Changing %s to %i", gpm_pref_key, checked);
-	gpm_conf_set_bool (prefs->priv->conf, gpm_pref_key, checked);
+	gconf_client_set_bool (prefs->priv->conf, gpm_pref_key, checked, NULL);
 }
 
 /**
@@ -582,7 +580,7 @@ gpm_prefs_setup_checkbox (GpmPrefs    *prefs,
 
 	widget = glade_xml_get_widget (xml, widget_name);
 
-	gpm_conf_get_bool (prefs->priv->conf, gpm_pref_key, &checked);
+	checked = gconf_client_get_bool (prefs->priv->conf, gpm_pref_key, NULL);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), checked);
 
 	g_object_set_data (G_OBJECT (widget), "conf_key", (gpointer) gpm_pref_key);
@@ -629,7 +627,7 @@ gpm_prefs_delete_event_cb (GtkWidget *widget,
  * @widget_name: The widget name
  *
  * Here we make sure that the start of the hscale is set to the
- * gnome-screensaver idle time to avoid confusion.
+ * gnome-session idle time to avoid confusion.
  **/
 static void
 set_idle_hscale_stops (GpmPrefs    *prefs,
@@ -639,60 +637,45 @@ set_idle_hscale_stops (GpmPrefs    *prefs,
 	GtkWidget *widget;
 	widget = glade_xml_get_widget (prefs->priv->glade_xml, widget_name);
 	if (gs_idle_time + 1 > NEVER_TIME_ON_SLIDER) {
-		egg_warning ("gnome-screensaver timeout is really big. "
-			     "Not sure what to do");
+		egg_warning ("gnome-session timeout is really big");
 		return;
 	}
 	gtk_range_set_range (GTK_RANGE (widget), gs_idle_time + 1, NEVER_TIME_ON_SLIDER);
 }
 
 /**
- * gs_delay_changed_cb:
- * @key: The conf key
- * @prefs: This prefs class instance
- **/
-static void
-gs_delay_changed_cb (GpmScreensaver *screensaver,
-		     gint	     delay,
-		     GpmPrefs	    *prefs)
-{
-	/* update the start and stop points on the hscales */
-	set_idle_hscale_stops (prefs, "hscale_battery_computer", delay);
-	set_idle_hscale_stops (prefs, "hscale_battery_display", delay);
-	set_idle_hscale_stops (prefs, "hscale_ac_computer", delay);
-	set_idle_hscale_stops (prefs, "hscale_ac_display", delay);
-}
-
-/**
- * conf_key_changed_cb:
+ * gpm_conf_gconf_key_changed_cb:
  *
- * We might have to do things when the conf keys change; do them here.
+ * We might have to do things when the gconf keys change; do them here.
  **/
 static void
-conf_key_changed_cb (GpmConf     *conf,
-		     const gchar *key,
-		     GpmPrefs    *prefs)
+gpm_conf_gconf_key_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, GpmPrefs *prefs)
 {
-	int value;
+	GConfValue *value;
+	gint brightness;
 	GtkWidget *widget;
-	gboolean  enabled;
+	gboolean enabled;
 
-	if (strcmp (key, GPM_CONF_BACKLIGHT_BRIGHTNESS_AC) == 0) {
+	value = gconf_entry_get_value (entry);
+	if (value == NULL)
+		return;
+
+	if (strcmp (entry->key, GPM_CONF_BACKLIGHT_BRIGHTNESS_AC) == 0) {
 		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hscale_ac_brightness");
-		gpm_conf_get_int (conf, key, &value);
-		gtk_range_set_value (GTK_RANGE (widget), value);
+		brightness = gconf_value_get_int (value);
+		gtk_range_set_value (GTK_RANGE (widget), brightness);
 	}
 
-	if (strcmp (key, GPM_CONF_LOWPOWER_AC) == 0) {
-		gpm_conf_get_bool (prefs->priv->conf, GPM_CONF_LOWPOWER_AC, &enabled);
+	if (strcmp (entry->key, GPM_CONF_LOWPOWER_AC) == 0) {
+		enabled = gconf_value_get_bool (value);
 		egg_debug ("need to enable checkbox");
 
-	} else if (strcmp (key, GPM_CONF_LOWPOWER_UPS) == 0) {
-		gpm_conf_get_bool (prefs->priv->conf, GPM_CONF_LOWPOWER_UPS, &enabled);
+	} else if (strcmp (entry->key, GPM_CONF_LOWPOWER_UPS) == 0) {
+		enabled = gconf_value_get_bool (value);
 		egg_debug ("need to enable checkbox");
 
-	} else if (strcmp (key, GPM_CONF_LOWPOWER_BATT) == 0) {
-		gpm_conf_get_bool (prefs->priv->conf, GPM_CONF_LOWPOWER_BATT, &enabled);
+	} else if (strcmp (entry->key, GPM_CONF_LOWPOWER_BATT) == 0) {
+		enabled = gconf_value_get_bool (value);
 		egg_debug ("need to enable checkbox");
 	}
 }
@@ -710,7 +693,7 @@ prefs_setup_notification (GpmPrefs *prefs)
 	GtkWidget   *radiobutton_icon_never;
 	gboolean     is_writable;
 
-	gpm_conf_get_string (prefs->priv->conf, GPM_CONF_UI_ICON_POLICY, &icon_policy_str);
+	icon_policy_str = gconf_client_get_string (prefs->priv->conf, GPM_CONF_UI_ICON_POLICY, NULL);
 	icon_policy = gpm_tray_icon_mode_from_string (icon_policy_str);
 	g_free (icon_policy_str);
 
@@ -725,7 +708,7 @@ prefs_setup_notification (GpmPrefs *prefs)
 	radiobutton_icon_never = glade_xml_get_widget (prefs->priv->glade_xml,
 							"radiobutton_notification_never");
 
-	gpm_conf_is_writable (prefs->priv->conf, GPM_CONF_UI_ICON_POLICY, &is_writable);
+	is_writable = gconf_client_key_is_writable (prefs->priv->conf, GPM_CONF_UI_ICON_POLICY, NULL);
 	gtk_widget_set_sensitive (radiobutton_icon_always, is_writable);
 	gtk_widget_set_sensitive (radiobutton_icon_present, is_writable);
 	gtk_widget_set_sensitive (radiobutton_icon_charge, is_writable);
@@ -790,7 +773,6 @@ static void
 prefs_setup_ac (GpmPrefs *prefs)
 {
 	GtkWidget *widget;
-	gint delay;
 	const gchar  *button_lid_actions[] =
 				{ACTION_NOTHING,
 				 ACTION_BLANK,
@@ -812,9 +794,8 @@ prefs_setup_ac (GpmPrefs *prefs)
 	gpm_prefs_setup_checkbox (prefs, "checkbutton_ac_display_dim",
 				  GPM_CONF_BACKLIGHT_IDLE_DIM_AC);
 
-	delay = gpm_screensaver_get_delay (prefs->priv->screensaver);
-	set_idle_hscale_stops (prefs, "hscale_ac_computer", delay);
-	set_idle_hscale_stops (prefs, "hscale_ac_display", delay);
+	set_idle_hscale_stops (prefs, "hscale_ac_computer", prefs->priv->idle_delay);
+	set_idle_hscale_stops (prefs, "hscale_ac_display", prefs->priv->idle_delay);
 
 	if (prefs->priv->has_button_lid == FALSE) {
 		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_ac_lid");
@@ -833,7 +814,6 @@ prefs_setup_battery (GpmPrefs *prefs)
 {
 	GtkWidget *widget;
 	GtkWidget *notebook;
-	gint delay;
 	gint page;
 
 	const gchar  *button_lid_actions[] =
@@ -881,9 +861,8 @@ prefs_setup_battery (GpmPrefs *prefs)
 		gtk_widget_hide_all (widget);
 	}
 
-	delay = gpm_screensaver_get_delay (prefs->priv->screensaver);
-	set_idle_hscale_stops (prefs, "hscale_battery_computer", delay);
-	set_idle_hscale_stops (prefs, "hscale_battery_display", delay);
+	set_idle_hscale_stops (prefs, "hscale_battery_computer", prefs->priv->idle_delay);
+	set_idle_hscale_stops (prefs, "hscale_battery_display", prefs->priv->idle_delay);
 
 	if (prefs->priv->has_button_lid == FALSE) {
 		widget = glade_xml_get_widget (prefs->priv->glade_xml, "hbox_battery_lid");
@@ -900,7 +879,6 @@ prefs_setup_ups (GpmPrefs *prefs)
 {
 	GtkWidget *widget;
 	GtkWidget *notebook;
-	gint delay;
 	gint page;
 
 	const gchar  *ups_low_actions[] =
@@ -925,8 +903,7 @@ prefs_setup_ups (GpmPrefs *prefs)
 				      ups_low_actions);
 	gpm_prefs_setup_sleep_slider (prefs, "hscale_ups_computer",
 				      GPM_CONF_TIMEOUT_SLEEP_COMPUTER_BATT);
-	delay = gpm_screensaver_get_delay (prefs->priv->screensaver);
-	set_idle_hscale_stops (prefs, "hscale_ups_computer", delay);
+	set_idle_hscale_stops (prefs, "hscale_ups_computer", prefs->priv->idle_delay);
 }
 
 static void
@@ -972,10 +949,11 @@ prefs_setup_general (GpmPrefs *prefs)
 static void
 pk_prefs_set_defaults_cb (PolKitGnomeAction *default_action, GpmPrefs *prefs)
 {
+	GConfClient *client;
 	DBusGProxy *proxy;
 	DBusGConnection *connection;
 	GError *error;
-	gchar *keys[5] = {
+	const gchar *keys[5] = {
 		"/apps/gnome-power-manager/actions",
 		"/apps/gnome-power-manager/ui",
 		"/apps/gnome-power-manager/buttons",
@@ -1000,7 +978,6 @@ pk_prefs_set_defaults_cb (PolKitGnomeAction *default_action, GpmPrefs *prefs)
 		return;
 	}
 
-	GConfClient *client;
 	client = gconf_client_get_default ();
 	gconf_client_suggest_sync (client, NULL);
 	g_object_unref (client);
@@ -1042,15 +1019,9 @@ gpk_prefs_setup_policykit (GpmPrefs *prefs)
 
 	/* set default */
 	pk_action = polkit_action_new ();
-	polkit_action_set_action_id (pk_action, "org.freedesktop.packagekit.set-defaults");
+	polkit_action_set_action_id (pk_action, "org.gnome.gconf.defaults.set-system");
 	prefs->priv->default_action = polkit_gnome_action_new_default ("set-defaults", pk_action,
 								       _("Make Default"), NULL);
-	g_object_set (prefs->priv->default_action,
-		      "no-icon-name", GTK_STOCK_FLOPPY,
-		      "auth-icon-name", GTK_STOCK_FLOPPY,
-		      "yes-icon-name", GTK_STOCK_FLOPPY,
-		      "self-blocked-icon-name", GTK_STOCK_FLOPPY,
-		      NULL);
 	polkit_action_unref (pk_action);
 }
 #endif
@@ -1068,13 +1039,16 @@ gpm_prefs_init (GpmPrefs *prefs)
 
 	prefs->priv = GPM_PREFS_GET_PRIVATE (prefs);
 
-	prefs->priv->screensaver = gpm_screensaver_new ();
-	g_signal_connect (prefs->priv->screensaver, "gs-delay-changed",
-			  G_CALLBACK (gs_delay_changed_cb), prefs);
+	prefs->priv->conf = gconf_client_get_default ();
+	/* watch gnome-power-manager keys */
+	gconf_client_add_dir (prefs->priv->conf, GPM_CONF_DIR,
+			      GCONF_CLIENT_PRELOAD_NONE, NULL);
+	gconf_client_notify_add (prefs->priv->conf, GPM_CONF_DIR,
+				 (GConfClientNotifyFunc) gpm_conf_gconf_key_changed_cb,
+				 prefs, NULL, NULL);
 
-	prefs->priv->conf = gpm_conf_new ();
-	g_signal_connect (prefs->priv->conf, "value-changed",
-			  G_CALLBACK (conf_key_changed_cb), prefs);
+	/* get value of delay in gnome-session */
+	prefs->priv->idle_delay = gconf_client_get_int (prefs->priv->conf, GPM_CONF_IDLE_DELAY, NULL);
 
 	caps = gpm_dbus_method_int ("GetPreferencesOptions");
 	prefs->priv->has_batteries = ((caps & GPM_PREFS_SERVER_BATTERY) > 0);
@@ -1083,7 +1057,8 @@ gpm_prefs_init (GpmPrefs *prefs)
 	prefs->priv->has_ambient = ((caps & GPM_PREFS_SERVER_AMBIENT) > 0);
 	prefs->priv->has_button_lid = ((caps & GPM_PREFS_SERVER_LID) > 0);
 	prefs->priv->has_button_suspend = TRUE;
-	prefs->priv->can_shutdown = gpm_dbus_method_bool ("CanShutdown");
+	/* TODO: use DeviceKit-power */
+	prefs->priv->can_shutdown = TRUE;
 	prefs->priv->can_suspend = gpm_dbus_method_bool ("CanSuspend");
 	prefs->priv->can_hibernate = gpm_dbus_method_bool ("CanHibernate");
 	egg_debug ("caps=%i", caps);
@@ -1101,12 +1076,11 @@ gpm_prefs_init (GpmPrefs *prefs)
 		g_error ("Cannot find 'gpm-prefs.glade'");
 	}
 
-	main_window = glade_xml_get_widget (prefs->priv->glade_xml, "window_preferences");
+	main_window = glade_xml_get_widget (prefs->priv->glade_xml, "dialog_preferences");
 
 	/* Hide window first so that the dialogue resizes itself without redrawing */
 	gtk_widget_hide (main_window);
-
-	gtk_window_set_icon_name (GTK_WINDOW(main_window), GPM_STOCK_APP_ICON);
+	gtk_window_set_default_icon_name (GPM_STOCK_APP_ICON);
 
 	/* Get the main window quit */
 	g_signal_connect (main_window, "delete_event",
@@ -1149,9 +1123,6 @@ gpm_prefs_finalize (GObject *object)
 	prefs->priv = GPM_PREFS_GET_PRIVATE (prefs);
 
 	g_object_unref (prefs->priv->conf);
-	if (prefs->priv->screensaver) {
-		g_object_unref (prefs->priv->screensaver);
-	}
 
 	G_OBJECT_CLASS (gpm_prefs_parent_class)->finalize (object);
 }
